package pps.core.space.service;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import pps.core.common.session.PpsUserSession;
import pps.core.common.utils.BaseUtils;
import pps.core.space.entity.SpaceInstitutionDetailEnt;
import pps.core.space.entity.SpaceInstitutionDetailView;
import pps.core.space.entity.SpaceInstitutionDurationEnt;
import pps.core.space.entity.SpaceInstitutionWellheadView;
import pps.core.space.mapper.SpaceInstitutionDetailMapper;
import pps.core.space.mapper.SpaceInstitutionDetailViewMapper;
import pps.core.space.mapper.SpaceInstitutionDurationMapper;
import pps.core.space.mapper.SpaceInstitutionWellheadViewMapper;
import pps.core.space.service.data.space_institution_detail.*;
import pps.core.space.service.data.space_institution_wellhead.CreateSpaceInstitutionWellheadInput;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.data.CustomQueryInput;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XPageResult;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;
import xstartup.error.XError;
import xstartup.feature.api.annotation.XApiAnonymous;
import xstartup.feature.api.annotation.XApiGet;
import xstartup.feature.api.annotation.XApiPost;
import xstartup.feature.mybatis.helper.XMapperHelper;
import xstartup.helper.XTransactionHelper;

import java.util.List;

/**
 * 基础间开配置模块
 *
 * @author ZWT
 * @date 2023/09/05
 */
@XService
@XText("基础间开配置模块")
public class SpaceInstitutionDetailService {

    /**
     * 基础间开配置--新增
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XServiceResult}
     */
    @XApiAnonymous
    @XApiPost
    @XText("基础间开配置--新增")
    public XServiceResult createSpaceInstitutionInstitution(XContext context, CreateSpaceInstitutionDetailInput input) {
        PpsUserSession session = new PpsUserSession();
        session.setId("123");
        session.setUserName("ceshi");
//        PpsUserSession session = context.getSession(PpsUserSession.class);
        return XTransactionHelper.begin(context, () -> {
            //新增间开配置
            SpaceInstitutionDetailMapper mapper = context.getBean(SpaceInstitutionDetailMapper.class);
            SpaceInstitutionDetailEnt entity = XCopyUtils.copyNewObject(input, SpaceInstitutionDetailEnt.class);
            BaseUtils.setBaseModelDefault(entity, session);
            mapper.insert(entity);
            String institutionId = entity.getId();
            List<CreateSpaceInstitutionWellheadInput> wellheadList = input.getWellheadList();
            if (CollUtil.isNotEmpty(wellheadList)) {
                List<SpaceInstitutionDurationEnt> durationList;
                SpaceInstitutionWellheadViewMapper wellheadMapper = context.getBean(SpaceInstitutionWellheadViewMapper.class);
                List<SpaceInstitutionWellheadView> wellheads = XCopyUtils.copyNewList(wellheadList, SpaceInstitutionWellheadView.class);
                for (SpaceInstitutionWellheadView wellhead : wellheads) {
                    wellhead.setInstitutionId(institutionId);
                    BaseUtils.setBaseModelDefault(wellhead, session);
                    wellheadMapper.insert(wellhead);
                    String wellheadId = wellhead.getWellheadId();
                    String configId = wellhead.getId();
                    durationList = wellhead.getDurationList();
                    if (CollUtil.isNotEmpty(durationList)) {
                        durationList.forEach(d -> {
                            d.setWellheadId(wellheadId);
                            d.setConfigId(configId);
                            d.setInstitutionId(institutionId);
                            BaseUtils.setBaseModelDefault(d, session);
                        });
                        BaseUtils.batchInsert(SpaceInstitutionDurationMapper.class, SpaceInstitutionDurationEnt.class, durationList);
                    }
                }
            }
            return XServiceResult.OK;
        });
    }

    @XApiAnonymous
    @XApiPost
    public XServiceResult updateSpaceInstitutionInstitution(XContext context, UpdateSpaceInstitutionDetailInput input) {
        SpaceInstitutionDetailMapper mapper = context.getBean(SpaceInstitutionDetailMapper.class);
        QueryWrapper<SpaceInstitutionDetailEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(SpaceInstitutionDetailEnt::getId, input.getId());
        SpaceInstitutionDetailEnt entity = mapper.selectOne(queryWrapper);
        if (entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        XCopyUtils.copyObject(input, entity);
        mapper.updateById(entity);
        return XServiceResult.OK;
    }

    @XApiAnonymous
    @XApiPost
    public XServiceResult deleteSpaceInstitutionInstitution(XContext context, DeleteSpaceInstitutionDetailInput input) {
        SpaceInstitutionDetailMapper mapper = context.getBean(SpaceInstitutionDetailMapper.class);
        QueryWrapper<SpaceInstitutionDetailEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(SpaceInstitutionDetailEnt::getId, input.getId());
        SpaceInstitutionDetailEnt entity = mapper.selectOne(queryWrapper);
        if (entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        mapper.deleteById(entity);
        return XServiceResult.OK;
    }

    @XApiAnonymous
    @XApiGet
    public XSingleResult<GetSpaceInstitutionDetailOutput> getSpaceInstitutionInstitution(XContext context, GetSpaceInstitutionDetailInput input) {
        SpaceInstitutionDetailMapper mapper = context.getBean(SpaceInstitutionDetailMapper.class);
        QueryWrapper<SpaceInstitutionDetailEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(SpaceInstitutionDetailEnt::getId, input.getId());
        SpaceInstitutionDetailEnt entity = mapper.selectOne(queryWrapper);
        if (entity == null) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetSpaceInstitutionDetailOutput output = new GetSpaceInstitutionDetailOutput();
        XCopyUtils.copyObject(entity, output);
        return XSingleResult.success(output);
    }

    @XApiAnonymous
    @XApiGet
    public XPageResult<QuerySpaceInstitutionDetailOutput> querySpaceInstitutionInstitution(XContext context, QuerySpaceInstitutionDetailInput input) {
        SpaceInstitutionDetailMapper mapper = context.getBean(SpaceInstitutionDetailMapper.class);
        QueryWrapper<SpaceInstitutionDetailEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().like(SpaceInstitutionDetailEnt::getId, input.getId())
                .eq(SpaceInstitutionDetailEnt::getIsDeleted, input.getIsDeleted())
                .like(SpaceInstitutionDetailEnt::getCreateById, input.getCreateById())
                .like(SpaceInstitutionDetailEnt::getCreateByName, input.getCreateByName())
                .eq(SpaceInstitutionDetailEnt::getCreateTime, input.getCreateTime())
                .like(SpaceInstitutionDetailEnt::getModifyById, input.getModifyById())
                .like(SpaceInstitutionDetailEnt::getModifyByName, input.getModifyByName())
                .eq(SpaceInstitutionDetailEnt::getModifyTime, input.getModifyTime())
                .like(SpaceInstitutionDetailEnt::getOuId, input.getOuId())
                .like(SpaceInstitutionDetailEnt::getLineId, input.getLineId())
                .like(SpaceInstitutionDetailEnt::getGridTypeKey, input.getGridTypeKey())
                .like(SpaceInstitutionDetailEnt::getInstitutionName, input.getInstitutionName())
                .eq(SpaceInstitutionDetailEnt::getInstitutionStartDate, input.getInstitutionStartDate())
                .eq(SpaceInstitutionDetailEnt::getInstitutionEndDate, input.getInstitutionEndDate())
                .eq(SpaceInstitutionDetailEnt::getOptimizeDeadline, input.getOptimizeDeadline())
                .eq(SpaceInstitutionDetailEnt::getOptimizeState, input.getOptimizeState())
                .eq(SpaceInstitutionDetailEnt::getIsCurrentBasic, input.getIsCurrentBasic())
                .eq(SpaceInstitutionDetailEnt::getStartInterval, input.getStartInterval());
        return XMapperHelper.query(mapper, input, queryWrapper, QuerySpaceInstitutionDetailOutput.class);
    }

    @XApiAnonymous
    @XApiGet
    public XPageResult<QuerySpaceInstitutionDetailOutput> dynamicQuerySpaceInstitutionInstitution(XContext context, CustomQueryInput input) {
        SpaceInstitutionDetailMapper mapper = context.getBean(SpaceInstitutionDetailMapper.class);
        return XMapperHelper.query(mapper, input, SpaceInstitutionDetailEnt.class, QuerySpaceInstitutionDetailOutput.class);
    }

    @XApiAnonymous
    @XApiGet
    public XSingleResult<GetSpaceInstitutionDetailViewOutput> getSpaceInstitutionInstitutionView(XContext context, GetSpaceInstitutionDetailViewInput input) {
        SpaceInstitutionDetailViewMapper mapper = context.getBean(SpaceInstitutionDetailViewMapper.class);
        SpaceInstitutionDetailView record = new SpaceInstitutionDetailView();
        XCopyUtils.copyObject(input, record);
        SpaceInstitutionDetailView view = mapper.selectOne(record);
        if (view == null) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetSpaceInstitutionDetailViewOutput output = new GetSpaceInstitutionDetailViewOutput();
        XCopyUtils.copyObject(view, output);
        return XSingleResult.success(output);
    }

    @XApiAnonymous
    @XApiGet
    public XPageResult<QuerySpaceInstitutionDetailViewOutput> querySpaceInstitutionInstitutionView(XContext context, QuerySpaceInstitutionDetailViewInput input) {
        SpaceInstitutionDetailViewMapper mapper = context.getBean(SpaceInstitutionDetailViewMapper.class);
        SpaceInstitutionDetailView record = new SpaceInstitutionDetailView();
        XCopyUtils.copyObject(input, record);
        PageHelper.startPage(input.getPage(), input.getLimit());
        List<SpaceInstitutionDetailView> list = mapper.selectList(record);
        PageInfo<SpaceInstitutionDetailView> pageInfo = new PageInfo<>(list);
        List<QuerySpaceInstitutionDetailViewOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QuerySpaceInstitutionDetailViewOutput.class);
        return XPageResult.success(outputs, input, pageInfo.getTotal());
    }

}
